# -*- coding: utf-8 -*-
"""
Created on Tue Dec 31 12:56:21 2019

@author: 45063883
"""

import math
import numpy as np

class hyperparameter():
    def initialise(alpha,first_moment,second_moment,epsilon,noise_g):
        return alpha,first_moment,second_moment,epsilon,noise_g


# -*- coding: utf-8 -*-
"""
Created on Mon Dec 30 14:48:18 2019

@author: 45063883
"""

import math
import numpy as np
import Abhilash1_optimizers.Activation as Activation
import Abhilash1_optimizers.hyperparameters as hyperparameters
import Abhilash1_optimizers.Moment_Initializer as Moment_Initializer
#Adamax varaiation of ADAM with L**p norm over L**2 norm(p->infinity)
class SGD():
    def __init__(alpha,b_1,b_2,epsilon,noise_g):
        return hyperparameters.hyperparameter.initialise(alpha,b_1,b_2,epsilon,noise_g)
    def init(m_t,v_t,t,theta):
        return Moment_Initializer.Moment_Initializer.initialize(m_t,v_t,t,theta)
    def SGD_optimizer(data,len_data,max_itr,alpha,b_1,b_2,epsilon,noise_g,act_func,scale):
        alpha,b_1,b_2,epsilon,noise_g=SGD.__init__(alpha,b_1,b_2,epsilon,noise_g)
        m_t,v_t,t,theta_0=SGD.init(0,0,0,0)
        final_weight_vector=[]
        for i in range(len_data):
            theta_0=data[i]
            for i in range(max_itr):
                t+=1
                
                if(act_func=="softPlus"):
                    g_t=Activation.Activation.softplus(theta_0)
                elif (act_func=="relu"):
                    g_t=Activation.Activation.relu(theta_0)
                elif (act_func=="elu"):
                    g_t=Activation.Activation.elu(theta_0,alpha)
                elif (act_func=="selu"):
                    g_t=Activation.Activation.selu(scale,theta_0,theta)
                elif (act_func=="tanh"):
                    g_t=Activation.Activation.tanh(theta_0)
                elif (act_func=="hardSigmoid"):
                    g_t=Activation.Activation.hard_sigmoid(theta_0)
                elif (act_func=="softSign"):
                    g_t=Activation.Activation.softsign(theta_0)
                elif (act_func=="linear"):
                    g_t=Activation.Activation.linear(theta_0)
                elif (act_func=="exponential"):
                    g_t=Activation.Activation.exponential(theta_0)
                
                theta_prev=theta_0
        
                alpha_t=alpha*(g_t)
                theta_0=theta_prev-(alpha_t)
                print("Intrermediate gradients")
                print("==========================================")
                print("Previous gradient",theta_prev)
                print("Present gradient",theta_0)
                print("==========================================")
                #if theta_0==theta_prev:
                    #    break;
                    
            final_weight_vector.append(theta_0)
            
        return final_weight_vector



    def initialize(data,max_itr):
        len_data=len(data)
        optimized_weights=SGD.SGD_optimizer(data,len_data,max_itr,alpha,b_1,b_2,epsilon,noise_g,act_func,scale)
        print("Optimized Weight Vector")
        print("=====================================")
        for i in range(len(optimized_weights)):
            print("=====",optimized_weights[i])

if __name__=='__main__':
    print("Verbose")
    #t_0=Adagrad_optimizer()
    #print("gradient coefficient",t_0)
    #solve_grad=poly_func(t_0)
    #print("Gradient Value",solve_grad)
    sample_data=[1,0.5,0.7,0.1]
    SGD.initialize(sample_data,100)    





# -*- coding: utf-8 -*-
"""
Created on Tue Dec 31 13:03:56 2019

@author: 45063883
"""

import math 
import numpy as np

class Moment_Initializer():
    def initialize(momentum_1,momentum_2,turns,theta):
        return momentum_1,momentum_2,turns,theta


# -*- coding: utf-8 -*-
"""
Created on Tue Dec 31 15:20:24 2019

@author: 45063883
"""

import Abhilash1_optimizers.ADAM_modified_update as ad

if __name__=="__main__":
    s=[0.8,0.9,0.7,0.67]
    max_itr=5
    ad.ADAMM.initialize(s,max_itr)


# -*- coding: utf-8 -*-
"""
Created on Tue Dec 31 16:47:43 2019

@author: 45063883
"""


from distutils.core import setup
setup(
  name = 'Abhilash1_optimizers',         
  packages = ['Abhilash1_optimizers'],   
  version = '0.1',       
  license='MIT',        
  description = 'Stochastic Optimizers for gradient convergence including activation functions',   
  author = 'ABHILASH MAJUMDER',
  author_email = 'abhilash.majumder@hsbc.co.in',
  url = 'https://github.com/abhilash1910/Abhilash1_optimizers_package',   
  download_url = 'https://github.com/abhilash1910/Abhilash1_optimizers_package/archive/v_01.tar.gz',    
  keywords = ['adam', 'adagrad', 'adamax','optimizers','stochastic','sgd','rmsprop','momentum','nesterov adam'],   
  install_requires=[           

          'numpy',         
          'pandas' 
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3',      
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)



# -*- coding: utf-8 -*-
"""
Created on Tue Dec 31 12:39:41 2019

@author: 45063883
"""

import math
import numpy as np

class Activation():
    def sigmoid(x):
        return (1.0/(1.0 + math.exp(-x)))
    def relu(x):
        return max(x,0)
    def elu(x,alp):
        if x<=0:
            return alp*(math.exp(x)-1)
        elif x>0:
            return x
    def selu(scale,x,alp):
        return scale*Activation.elu(x,alp)
    def tanh(x):
        return ((math.exp(x)-math.exp(-x))/(math.exp(x)+math.exp(-x)))
    def hard_sigmoid(x):
        if x>2.5:
            return 1
        elif x<(-2.5):
            return 0
        elif(x>=(-2.5) and x<=(2.5)):
            return (0.2*x +0.5)
    def exponential(x):
        return math.exp(x)
    def linear(x):
        return x
    def softsign(x):
        return (1.0/math.abs(x)+1.0)
    def softplus(x):
        return math.log(math.exp(x)+1)

    